package com.fr.plugin.yxSSO.filter;

import com.fr.decision.fun.impl.AbstractEmbedRequestFilterProvider;
import com.fr.json.JSONObject;
import com.fr.plugin.yxSSO.config.simple.account.PluginSimpleConfig;
import com.fr.plugin.yxSSO.utils.FRUtils;
import com.fr.plugin.yxSSO.utils.HttpUtils;
import com.fr.plugin.yxSSO.utils.ResponseUtils;
import com.fr.plugin.yxSSO.utils.Utils;
import sun.net.www.http.HttpClient;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

public class SSOFilter extends AbstractEmbedRequestFilterProvider {

    @Override
    public void filter(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        boolean isLogin = FRUtils.isLogin(httpServletRequest);
        String url = FRUtils.getAllUrl(httpServletRequest);

        boolean isMobile = Utils.isMobile(httpServletRequest);

        //如果不是手机端链接则放行
        if(!isMobile){
            return;
        }

        //如果已经登录则放行
        if(isLogin){
            return ;
        }

        //如果是自带登录页资源则放行
        if(url.contains("login")||url.contains("decision/file")||url.contains("decision/resource")||url.contains("decision/system")||url.contains("query/ip")){
            return;
        }

        //如果是远程设计则放行

        if(url.contains("remote")){
            return;
        }

        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String clientId = psc.getClientId();
        String secret = psc.getSecret();
        //判断有无code
        //无跳转登录页面
        String code = httpServletRequest.getParameter("code");

        if(Utils.isNullStr(code)){
            String redirect_uri = "";
            try {
                redirect_uri = URLEncoder.encode(url,"UTF-8");
            } catch (UnsupportedEncodingException e) {
                ResponseUtils.failedResponse(httpServletResponse,"url编码异常");
                return;
            }

            String authUrl = psc.getAuthUrl();
            authUrl+="?client_id="+clientId+"&response_type=code&scope=user&state=1&redirect_uri="+redirect_uri;
            FRUtils.FRLogInfo("authUrl:"+authUrl);
            try {
                httpServletResponse.sendRedirect(authUrl);
            } catch (IOException e) {
                ResponseUtils.failedResponse(httpServletResponse,"重定向异常");
                return;
            }
        }

        //有进行信息验证
        //获取token
        String tokenUrl = psc.getTokenUrl();

        String redirect_uri = url.contains("?code") ? url.substring(0,url.indexOf("?code")): url.substring(0,url.indexOf("&code"));
        tokenUrl += "?grant_type=authorization_code&code="+code+"&redirect_uri="+redirect_uri+"&client_id="+clientId+"&client_secret="+secret;

        String tokenResult = HttpUtils.HttpPostJson(tokenUrl,"",null);

        if(Utils.isNullStr(tokenResult)){
            ResponseUtils.failedResponse(httpServletResponse,"获取token失败");
            return;
        }

        JSONObject tokenJson = new JSONObject(tokenResult);
        String token = tokenJson.getString("access_token");
        //获取用户信息
        String userinfoUrl = psc.getUserUrl();
        userinfoUrl += "?access_token="+token;
        String userinfoResult = HttpUtils.HttpPostJson(userinfoUrl,"",null);

        if(Utils.isNullStr(userinfoResult)){
            ResponseUtils.failedResponse(httpServletResponse,"获取用户信息失败");
            return;
        }

        JSONObject userJson = new JSONObject(userinfoResult);
        String username = userJson.getString("login_name");
        //登录
        FRUtils.login(httpServletRequest,httpServletResponse,username,"");
    }
}